import RPi.GPIO as GPIO
import subprocess
import speech_recognition as sr
from gtts import gTTS
from googletrans import Translator
from utils import update_display
import time

# Setup for GPIO buttons
BUTTON_PIN_EN_TO_BN = 17  # English to Bengali
BUTTON_PIN_BN_TO_EN = 22  # Bengali to English

GPIO.setmode(GPIO.BCM)
GPIO.setup(BUTTON_PIN_EN_TO_BN, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(BUTTON_PIN_BN_TO_EN, GPIO.IN, pull_up_down=GPIO.PUD_UP)

recording = False
process = None

# Function to record audio
def record_audio(filename):
    command = ["arecord", "-D", "plughw:1,0", "-f", "S16_LE", "-r", "16000", filename]
    return subprocess.Popen(command)

# Function for speech-to-text
def speech_to_text(filename, src_lang="en"):
    recognizer = sr.Recognizer()
    try:
        with sr.AudioFile(filename) as source:
            audio_data = recognizer.record(source)
            text = recognizer.recognize_google(audio_data, language=src_lang)
            return text
    except Exception as e:
        print(f"Error recognizing speech: {e}")
        return ""

# Function to convert text to target language and speak it
def text_to_speech(text, src_lang, dest_lang):
    translator = Translator()
    translated_text = translator.translate(text, src=src_lang, dest=dest_lang).text
    tts = gTTS(text=translated_text, lang=dest_lang)
    tts.save(f"output_{dest_lang}.mp3")
    subprocess.run(["mpg123", "-q", f"output_{dest_lang}.mp3"], stderr=subprocess.DEVNULL)

# Callback for English to Bengali
def button_callback_en_to_hi(channel):
    global recording, process
    if not recording:
        print("Recording started (English to Hindi)...")
        update_display("Recording started (EN to HI)")
        process = record_audio("speech.wav")
        recording = True
    else:
        print("Stopping recording...")
        update_display("Stopping recording")
        if process:
            process.terminate()
        recording = False
        print("Converting speech to text...")
        speech = speech_to_text("speech.wav", src_lang="en")
        print(f"Recognized speech: {speech}")
        update_display(f"Speech: {speech}")
        print("Converting to Hindi...")
        update_display("Converting to Hindi")
        text_to_speech(speech, src_lang="en", dest_lang="hi")

# Callback for Hindi to English
def button_callback_hi_to_en(channel):
    global recording, process
    if not recording:
        print("Recording started (Hindi to English)...")
        update_display("Recording started (HI to EN)")
        process = record_audio("speech.wav")
        recording = True
    else:
        print("Stopping recording...")
        update_display("Stopping recording")
        if process:
            process.terminate()
        recording = False
        print("Converting speech to text...")
        speech = speech_to_text("speech.wav", src_lang="hi")
        print(f"Recognized speech: {speech}")
        update_display(f"Speech: {speech}")
        print("Converting to English...")
        update_display("Converting to English")
        text_to_speech(speech, src_lang="hi", dest_lang="en")

# Setup event detection for both buttons
GPIO.add_event_detect(BUTTON_PIN_EN_TO_BN, GPIO.FALLING, callback=button_callback_en_to_hi, bouncetime=300)
GPIO.add_event_detect(BUTTON_PIN_BN_TO_EN, GPIO.FALLING, callback=button_callback_hi_to_en, bouncetime=300)

# Main loop to keep the program running
try:
    while True:
        time.sleep(0.1)
       # pass  # Infinite loop to keep the program running and waiting for button events
except KeyboardInterrupt:
    GPIO.cleanup()

