from gtts import gTTS
import subprocess
import speech_recognition as sr
import time
import Adafruit_GPIO.SPI as SPI
import Adafruit_ILI9341 as TFT
from PIL import Image, ImageDraw, ImageFont

# TFT Display setup
DC = 24
RST = 25
SPI_PORT = 0
SPI_DEVICE = 0

# Create TFT LCD display class for 240x320
disp = TFT.ILI9341(DC, rst=RST, spi=SPI.SpiDev(SPI_PORT, SPI_DEVICE, max_speed_hz=64000000))

# Initialize display
disp.begin()

# Function to update the display
def update_display(text):
    width = 240
    height = 320
    image = Image.new("RGB", (width, height))  # Create blank image for drawing
    draw = ImageDraw.Draw(image)
    draw.rectangle((0, 0, width, height), outline=0, fill=(255, 255, 220))  # Light background
    font = ImageFont.truetype("/usr/share/fonts/truetype/dejavu/DejaVuSans.ttf", 16)  # Load a TTF font

    # Text wrapping
    max_width = width - 12
    lines = []
    words = text.split(' ')
    line = ''
    for word in words:
        line_width = font.getbbox(line + word)[2] if line else font.getbbox(word)[2]
        if line_width <= max_width:
            line += word + ' '
        else:
            lines.append(line)
            line = word + ' '
    lines.append(line)

    y = 5
    for line in lines:
        draw.text((5, y), line, font=font, fill="#0000FF")  # Blue text
        y += font.getbbox(line)[3] - font.getbbox(line)[1]

    # Add current date and time at the bottom
    date_time = time.strftime("%Y-%m-%d %H:%M:%S")
    draw.text((5, height - 25), date_time, font=font, fill="#0000FF")

    rotated_image = image.rotate(180, expand=True)  # Rotate the image 180 degrees
    disp.display(rotated_image)  # Display image

# Function for Text to Speech (TTS)
def text_to_speech(text):
    tts = gTTS(text=text, lang="en")
    tts.save("output.mp3")
    subprocess.run(["mpg123", "-q", "output.mp3"], stderr=subprocess.DEVNULL)

# Function to record audio
def record_audio(file_name, duration=10):
    command = ["arecord", "-D", "plughw:1,0", "-f", "S16_LE", "-r", "16000", "-d", str(duration), file_name]
    subprocess.run(command, stderr=subprocess.DEVNULL)

# Function for Speech to Text (STT)
def speech_to_text(file_name):
    recognizer = sr.Recognizer()
    try:
        with sr.AudioFile(file_name) as source:
            audio_data = recognizer.record(source)
            text = recognizer.recognize_google(audio_data)
            return text.lower()
    except sr.UnknownValueError:
        print("Sorry, could not understand the audio (UnknownValueError).")
        return ""
    except sr.RequestError as e:
        print(f"Could not request results from Google Speech Recognition service; {e}")
        return ""
    except Exception as e:
        print(f"An error occurred during speech recognition: {e}")
        return ""
